/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.metadocument;

import static io.vertx.core.json.JsonObject.*;

import java.io.IOException;
import java.util.Optional;
import java.util.Set;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;

import org.apache.http.util.EntityUtils;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.common.logging.OzgCloudLogging;
import de.ozgcloud.zufi.common.HttpMethod;
import io.vertx.core.json.JsonObject;
import lombok.extern.log4j.Log4j2;

@ApplicationScoped
@OzgCloudLogging
@Log4j2
class MetaDocumentRepository {
	private static final int HTTP_CREATED = 201;
	private static final int HTTP_OK = 200;
	private static final String HITS_FIELD = "hits";
	static final String INDEX_TEMPLATE = "/%s";
	static final String INDEX_INSERT_TEMPLATE = INDEX_TEMPLATE + "/_create/%s";
	static final String INDEX_UPDATE_TEMPLATE = INDEX_TEMPLATE + "/_doc/%s";
	static final String SEARCH_TEMPLATE = INDEX_TEMPLATE + "/_search";
	static final String SEARCH_BODY = "{\"query\": {\"match\": { \"objectType\": \"" + MetaDocument.class.getName() + "\"}}}";

	@Inject
	RestClient restClient;

	@ConfigProperty(name = "ozgcloud.zufi.index.name", defaultValue = "zufi-manager")
	String indexName;

	public MetaDocument insert(MetaDocument document) {
		try {
			var response = restClient.performRequest(createInsertRequest(document));
			isResponseValid(response);
		} catch (IOException e) {
			throw new TechnicalException("Error inserting MetaDocument.", e);
		}

		return document;
	}

	Request createInsertRequest(MetaDocument document) {
		return createRequest(document, INDEX_INSERT_TEMPLATE);
	}

	public MetaDocument update(MetaDocument document) {
		try {
			var response = restClient.performRequest(createUpdateRequest(document));
			isResponseValid(response);
		} catch (IOException e) {
			throw new TechnicalException("Error updating MetaDocument.", e);
		}
		return document;
	}

	Request createUpdateRequest(MetaDocument document) {
		return createRequest(document, INDEX_UPDATE_TEMPLATE);
	}

	private Request createRequest(MetaDocument document, String template) {
		var request = new Request(HttpMethod.PUT,
				template.formatted(indexName, document.getId()));
		request.setJsonEntity(mapFrom(document).toString());
		return request;
	}

	public Optional<MetaDocument> find() {
		try {
			return processResponse(getFindResponse());
		} catch (IOException e) {
			throw new TechnicalException("Error loading the MetaDocument", e);
		}
	}

	private String getFindResponse() throws IOException {
		var request = new Request(HttpMethod.GET, SEARCH_TEMPLATE.formatted(indexName));
		request.setJsonEntity(SEARCH_BODY);
		return EntityUtils.toString(restClient.performRequest(request).getEntity());
	}

	private Optional<MetaDocument> processResponse(String responseBody) {
		var searchHits = new JsonObject(responseBody).getJsonObject(HITS_FIELD).mapTo(SearchHits.class);
		if (searchHits.getTotal().value() > 0) {
			return Optional.of(searchHits.getHits()[0]._source());
		} else {
			return Optional.empty();
		}
	}

	private void isResponseValid(Response response) {
		Integer code = response.getStatusLine().getStatusCode();
		if (Set.of(HTTP_CREATED, HTTP_OK).contains(code)) {
			LOG.debug("MetaDocument saved.");
		} else {
			LOG.error("Error inserting/updating MetaDocument, Elasticsearch server returned Status {} with reason {}",
					code, response.getStatusLine().getReasonPhrase());
		}
	}
}
