/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static io.vertx.core.json.JsonObject.*;

import java.io.IOException;
import java.util.Set;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;

import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.common.logging.OzgCloudLogging;
import de.ozgcloud.zufi.common.HttpMethod;
import lombok.extern.log4j.Log4j2;

@ApplicationScoped
@OzgCloudLogging
@Log4j2
class OrganisationsEinheitRepository {
	private static final int HTTP_CREATED = 201;
	private static final int HTTP_OK = 200;
	static final String INDEX_TEMPLATE = "/%s";
	static final String INDEX_UPSERT_TEMPLATE = INDEX_TEMPLATE + "/_doc/%s";
	static final String DELETE_ENTRY_TEMPLATE = INDEX_TEMPLATE + "/_delete_by_query";
	static final String DELETE_BODY_TEMPLATE = "{\"query\": {\"match\": {\"id\": \"%s\"}}}";

	@Inject
	RestClient elasticClient;

	@ConfigProperty(name = "ozgcloud.zufi.index.name", defaultValue = "zufi-manager")
	String indexName;

	public OrganisationsEinheit upsert(OrganisationsEinheit organisationsEinheit) {
		try {
			var response = elasticClient.performRequest(createInsertRequest(organisationsEinheit));
			isResponseValid(response, HTTP_CREATED, HTTP_OK);
		} catch (IOException e) {
			throw new TechnicalException("Error inserting OrganisationsEinheit.", e);
		}

		return organisationsEinheit;
	}

	Request createInsertRequest(OrganisationsEinheit organisationsEinheit) {
		var request = new Request(HttpMethod.POST,
				INDEX_UPSERT_TEMPLATE.formatted(indexName, organisationsEinheit.getId()));
		request.setJsonEntity(mapFrom(organisationsEinheit).toString());
		return request;
	}

	public void delete(String id) {
		try {
			var response = elasticClient.performRequest(createDeleteRequest(id));
			isResponseValid(response, HTTP_OK);
		} catch (IOException e) {
			throw new TechnicalException("Error deleting OrganisationsEinheit.", e);
		}
	}

	Request createDeleteRequest(String id) {
		var request = new Request(HttpMethod.POST, DELETE_ENTRY_TEMPLATE.formatted(indexName));
		request.setJsonEntity(DELETE_BODY_TEMPLATE.formatted(id));
		return request;

	}

	private void isResponseValid(Response response, Integer... expected) {
		Integer code = response.getStatusLine().getStatusCode();
		if (Set.of(expected).contains(code)) {
			LOG.debug("Received valid response {}", response);
		} else {
			LOG.error("Error handling OrganisationsEinheit, Elasticsearch server returned Status {} with message {}",
					code, response.getStatusLine().getReasonPhrase());
		}
	}
}
