/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import javax.xml.bind.JAXBException;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.common.logging.OzgCloudLogging;
import de.ozgcloud.zufi.metadocument.MetaDocumentService;
import de.ozgcloud.zufi.organisationseinheit.OrganisationsEinheitService;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;

@ApplicationScoped
@OzgCloudLogging
@Log4j2
public class PvogProcessor {
	static final String INVALID_DATA_RECEICED_FROM_PVOG_SERVICE = "Invalid data receiced from PVOG Service";
	static final String GENERAL_ERROR_PROCESSING_PVOG_DATA = "General error processing PVOG data";

	@Inject
	PvogService pvogService;

	@Inject
	MetaDocumentService metaDocumentService;

	@Inject
	OrganisationsEinheitService organisationsEinheitService;

	@SneakyThrows
	public void process() {
		var startIndex = metaDocumentService.findNaechsterIndex();

		startIndex.ifPresentOrElse(
				index -> {
					LOG.info("Start incremental sync with index {}", index);
					handleAntwortTransfernachricht(pvogService.getIncrementVerwaltungsObjekte(index));
				},
				() -> {
					LOG.info("Start full sync");
					handleAntwortTransfernachricht(pvogService.getAllVerwaltungsObjekte());
				});
		LOG.info("Finished sync");
	}

	void handleAntwortTransfernachricht(AntwortTransfernachricht transfernachricht) {
		if (pvogService.dataAvailable(transfernachricht)) {
			try {
				doProcessing(transfernachricht);
			} catch (JAXBException e) {
				throw new TechnicalException(INVALID_DATA_RECEICED_FROM_PVOG_SERVICE, e);
			} catch (Exception e) {
				throw new TechnicalException(GENERAL_ERROR_PROCESSING_PVOG_DATA, e);
			}
		}
	}

	private void doProcessing(AntwortTransfernachricht transfernachricht) throws JAXBException {
		processTransferOperations(pvogService.parseXml(transfernachricht.xzufiObjekte()));

		metaDocumentService.saveMetaDocument(transfernachricht);

		if (pvogService.hasMoreData(transfernachricht)) {
			LOG.info("Fetching further data using naechsterIndex {}", transfernachricht.naechsterIndex());
			handleAntwortTransfernachricht(pvogService.getIncrementVerwaltungsObjekte(transfernachricht.naechsterIndex()));
		}
	}

	void processTransferOperations(TransferOperation op) {
		op.getEntries().stream().forEach(entry -> organisationsEinheitService.process(entry));
	}
}
