/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import org.apache.commons.collections4.CollectionUtils;
import org.mapstruct.Mapper;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.zufi.common.OperationType;
import de.ozgcloud.zufi.common.PayloadType;
import de.ozgcloud.zufi.schema.Identifikator;
import de.ozgcloud.zufi.schema.LoescheObjekt;
import de.ozgcloud.zufi.schema.Organisationseinheit;
import de.ozgcloud.zufi.schema.SchreibeObjekt;
import de.ozgcloud.zufi.schema.StringLocalized;

@Mapper(unmappedTargetPolicy = ReportingPolicy.WARN)
interface TransferOperationEntryMapper {
	default TransferOperationEntry mapTo(Object entry) {
		if (entry instanceof SchreibeObjekt schreibeObjekt) {
			return mapSchreibeObjekt(schreibeObjekt);
		} else if (entry instanceof LoescheObjekt loescheObject) {
			return TransferOperationEntry.builder()
					.operationType(OperationType.DELETE)
					.payload(Map.of(TransferOperationEntry.ID_FIELD, mapId(loescheObject.getId()))).build();
		} else {
			return TransferOperationEntry.builder().operationType(OperationType.UNKNOWN).build();
		}
	}

	private TransferOperationEntry mapSchreibeObjekt(SchreibeObjekt schreibeObjekt) {
		TransferOperationEntry.TransferOperationEntryBuilder builder = TransferOperationEntry.builder();
		builder.operationType(OperationType.WRITE);

		if (Objects.nonNull(schreibeObjekt.getOrganisationseinheit())) {
			var dataMap = new HashMap<>(mapOrganisationsEinheit(schreibeObjekt.getOrganisationseinheit()));
			dataMap.put(TransferOperationEntry.ID_FIELD, mapId(schreibeObjekt.getOrganisationseinheit().getId()));
			builder.payload(Collections.unmodifiableMap(dataMap));
			builder.payloadType(PayloadType.ORGANISATIONSEINHEIT);
		} else {
			builder.payloadType(PayloadType.UNKNOWN);
		}

		return builder.build();
	}

	private Map<String, String> mapOrganisationsEinheit(Organisationseinheit organisationseinheit) {
		var map = new HashMap<String, String>();
		mapName(organisationseinheit).ifPresent(name -> map.put(TransferOperationEntry.ORGANISATIONSEINHEIT_NAME_FIELD, name));
		mapAnschrift(organisationseinheit).ifPresent(map::putAll);
		return map;
	}

	private String mapId(Identifikator identificator) {
		return identificator.getValue();
	}

	private Optional<String> mapName(Organisationseinheit organisation) {
		Optional<StringLocalized> name = Optional.empty();
		if (hasName(organisation)) {
			name = Optional.ofNullable(organisation.getName().get(0).getName().get(0));
		}

		return name.map(StringLocalized::getValue);
	}

	private boolean hasName(Organisationseinheit organisation) {
		return Objects.nonNull(organisation.getName())
				&& CollectionUtils.isNotEmpty(organisation.getName());
	}

	private Optional<Map<String, String>> mapAnschrift(Organisationseinheit organisationseinheit) {
		Optional<Map<String, String>> anschriftMap = Optional.empty();
		if (hasAnschrift(organisationseinheit)) {
			var code = organisationseinheit.getAnschrift().get(0).getVerwaltungspolitischeKodierung();

			var map = new HashMap<String, String>();
			Optional.ofNullable(code.getBezirk())
					.ifPresent(value -> map.put(TransferOperationEntry.ORGANISATIONSEINHEIT_BEZIRK_FIELD, value.getName()));
			Optional.ofNullable(code.getBundesland())
					.ifPresent(value -> map.put(TransferOperationEntry.ORGANISATIONSEINHEIT_BUNDESLAND_FIELD, value.getName()));
			Optional.ofNullable(code.getGemeindeschluessel())
					.ifPresent(value -> map.put(TransferOperationEntry.ORGANISATIONSEINHEIT_GEMEINDESCHLUESSEL_FIELD, value.getName()));
			Optional.ofNullable(code.getGemeindeteilschluessel())
					.ifPresent(value -> map.put(TransferOperationEntry.ORGANISATIONSEINHEIT_GEMEINDETEILSCHLUESSEL_FIELD, value.getName()));
			Optional.ofNullable(code.getKreis())
					.ifPresent(value -> map.put(TransferOperationEntry.ORGANISATIONSEINHEIT_KREIS_FIELD, value.getName()));
			Optional.ofNullable(code.getRegionalschluessel())
					.ifPresent(value -> map.put(TransferOperationEntry.ORGANISATIONSEINHEIT_REGIONALSCHLUESSEL_FIELD, value.getName()));

			return Optional.of(Collections.unmodifiableMap(map));
		}

		return anschriftMap;
	}

	private boolean hasAnschrift(Organisationseinheit organisationseinheit) {
		return Objects.nonNull(organisationseinheit.getAnschrift())
				&& CollectionUtils.isNotEmpty(organisationseinheit.getAnschrift());
	}
}
