/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.search;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;

import javax.net.ssl.SSLContext;

import org.apache.http.impl.nio.client.HttpAsyncClientBuilder;
import org.apache.http.ssl.SSLContexts;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.elasticsearch.client.RestClientBuilder;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.common.logging.OzgCloudLogging;
import io.quarkus.elasticsearch.restclient.lowlevel.ElasticsearchClientConfig;

@ElasticsearchClientConfig
@OzgCloudLogging
public class SSLContextConfigurator implements RestClientBuilder.HttpClientConfigCallback {

	@ConfigProperty(name = "elasticsearch.ca.cert", defaultValue = "/elasticsearch-root-certificate/ca.crt")
	String caCrt;

	@ConfigProperty(name = "quarkus.elasticsearch.protocol")
	String elasticsearchProtocol;

	@Override
	public HttpAsyncClientBuilder customizeHttpClient(HttpAsyncClientBuilder builder) {
		if (isHttps()) {
			builder.setSSLContext(buildSslContext());
		}
		return builder;
	}

	private boolean isHttps() {
		return elasticsearchProtocol.equals("https");
	}

	SSLContext buildSslContext() {
		try {
			return SSLContexts.custom().loadTrustMaterial(buildKeyStore(), null).build();
		} catch (Exception e) {
			throw new TechnicalException("Error on building ssl context", e);
		}
	}

	KeyStore buildKeyStore() {
		try {
			var trustStore = initKeyStore();
			trustStore.setCertificateEntry("ca", generateCertificate());
			return trustStore;
		} catch (Exception e) {
			throw new TechnicalException("Error building KeyStore", e);
		}
	}

	KeyStore initKeyStore() {
		try {
			var trustStore = KeyStore.getInstance("pkcs12");
			trustStore.load(null, null);
			return trustStore;
		} catch (Exception e) {
			throw new TechnicalException("Error init KeyStore", e);
		}
	}

	Certificate generateCertificate() {
		try {
			return CertificateFactory.getInstance("X.509").generateCertificate(loadCaCrt());
		} catch (Exception e) {
			throw new TechnicalException("Error generating Certificate", e);
		}
	}

	InputStream loadCaCrt() {
		try {
			return Files.newInputStream(Paths.get(caCrt));
		} catch (IOException e) {
			throw new TechnicalException("Error loading caCert", e);
		}
	}
}