/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.metadocument;

import static org.assertj.core.api.Assertions.*;

import java.io.IOException;
import java.util.Date;

import jakarta.inject.Inject;

import org.elasticsearch.client.Request;
import org.elasticsearch.client.RestClient;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.zufi.common.HttpMethod;
import io.quarkus.test.junit.QuarkusTest;
import lombok.SneakyThrows;

@QuarkusTest
class MetaDocumentRepositoryITCase {
	private static final String INDEX_PATH = "/zufi-manager";
	private static final String REFRESH_PATH = INDEX_PATH + "/_refresh";
	@Inject
	MetaDocumentRepository metaDocumentRepository;

	@Inject
	RestClient restClient;

	@DisplayName("Test saving the MetaDocument")
	@Nested
	class TestSave {
		@DisplayName("insert a new MetaDocument")
		@Nested
		class InsertMetaDocument {
			@BeforeEach
			void initIndex() {
				deleteIndex();
				createIndex();
			}

			@Test
			void shouldInsert() {
				var doc = metaDocumentRepository.insert(MetaDocumentTestFactory.create());

				assertThat(doc).isNotNull();
			}

			@Test
			void shouldThrowException() {
				deleteIndex();
				var doc = MetaDocumentTestFactory.create();
				metaDocumentRepository.insert(doc);

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> metaDocumentRepository.insert(doc));
			}
		}

		@DisplayName("update an existing MetaDocument")
		@Nested
		class UpdateMetaDocument {
			@Test
			void shouldUpdate() {
				var newDoc = metaDocumentRepository.update(MetaDocumentTestFactory.create());
				refresh();
				var now = new Date().getTime();
				var updatedDoc = metaDocumentRepository
						.update(newDoc.toBuilder().lastRead(now).lastReadComplete(false).build());

				assertThat(updatedDoc).usingRecursiveComparison().ignoringFields("lastRead", "lastReadComplete").isEqualTo(newDoc);
				assertThat(updatedDoc.getLastRead()).isEqualTo(now);
				assertThat(updatedDoc.isLastReadComplete()).isFalse();
			}
		}
	}

	@DisplayName("Test loading the MetaDocument")
	@Nested
	class TestGet {
		@DisplayName("Load the MetaDocument")
		@Nested
		class GetMetaDocument {
			MetaDocument doc;

			@BeforeEach
			void initData() {
				deleteIndex();
				createIndex();

				doc = metaDocumentRepository.insert(MetaDocumentTestFactory.create());
				refresh();
			}

			@Test
			void shouldGet() {
				var foundDoc = metaDocumentRepository.find();

				assertThat(foundDoc).isPresent().get().usingRecursiveComparison().isEqualTo(doc);
			}
		}
	}

	@SneakyThrows
	void refresh() {
		restClient.performRequest(new Request(HttpMethod.POST, REFRESH_PATH));
	}

	void deleteIndex() {
		try {
			restClient.performRequest(new Request(HttpMethod.DELETE, INDEX_PATH));
		} catch (Exception e) {
			// ignore exception because the index might not exist yet
		}
	}

	void createIndex() {
		try {
			restClient.performRequest(new Request(HttpMethod.PUT, INDEX_PATH));
		} catch (IOException e) {
			// ignore exception when the index exists already
		}
	}
}
