/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.metadocument;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.ResponseException;
import org.elasticsearch.client.RestClient;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.util.ReflectionTestUtils;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.zufi.common.HttpMethod;
import lombok.SneakyThrows;

class MetaDocumentRepositoryTest {
	private static final String CONTENT = TestUtils.loadTextFile("findResponse.json");
	private static final String EMPTY_CONTENT = TestUtils.loadTextFile("emptyFindResponse.json");
	private static final String ZUFI_MANAGER = "zufi-manager";
	private static final int HTTP_CREATED = 201;
	private static final int HTTP_OK = 200;

	@Spy
	@InjectMocks
	MetaDocumentRepository repository;

	@Mock
	RestClient restClient;

	@Mock
	Response response;

	@Mock
	StatusLine statusLine;

	@Mock
	HttpEntity httpEntity;

	@DisplayName("Test performing requests")
	@Nested
	class TestRequests {

		@DisplayName("check updates and inserts")
		@Nested
		class VerifyInsertAndUpdateRequests {

			@DisplayName("when updating a new MetaDocument")
			@Nested
			class VerifyPerformingUpdateRequest {
				@BeforeEach
				void init() {
					mockResponse(HTTP_OK);
				}

				@Test
				@SneakyThrows
				void shouldCallRestClient() {
					repository.update(MetaDocumentTestFactory.create());

					verify(restClient).performRequest(any());
					verify(statusLine, never()).getReasonPhrase();
				}

				@Test
				@SneakyThrows
				void shouldHandleUnexpectedStatusCode() {
					mockResponse(304);

					repository.update(MetaDocumentTestFactory.create());

					verify(restClient).performRequest(any());
					verify(statusLine).getReasonPhrase();
				}
			}

			@DisplayName("when inserting a new MetaDocument")
			@Nested
			class VerifyPerformingInsertRequest {
				@BeforeEach
				void init() {
					mockResponse(HTTP_CREATED);
				}

				@Test
				@SneakyThrows
				void shouldCallRestClient() {
					repository.insert(MetaDocumentTestFactory.create());

					verify(restClient).performRequest(any());
					verify(statusLine, never()).getReasonPhrase();
				}

				@Test
				@SneakyThrows
				void shouldHandleUnexpectedStatusCode() {
					mockResponse(304);

					repository.insert(MetaDocumentTestFactory.create());

					verify(restClient).performRequest(any());
					verify(statusLine).getReasonPhrase();
				}
			}

			@SneakyThrows
			private void mockResponse(int status) {
				when(statusLine.getStatusCode()).thenReturn(status);
				when(response.getStatusLine()).thenReturn(statusLine);
				when(restClient.performRequest(any())).thenReturn(response);
			}
		}

		@DisplayName("when loading the MetaDocument")
		@Nested
		class VerifyPerformingSearchRequest {
			@BeforeEach
			@SneakyThrows
			void init() {
				when(httpEntity.getContentLength()).thenReturn(Long.valueOf(CONTENT.length()));
				when(httpEntity.getContent()).thenReturn(new ByteArrayInputStream(CONTENT.getBytes()));
				when(response.getEntity()).thenReturn(httpEntity);
				when(restClient.performRequest(any())).thenReturn(response);
			}

			@Test
			@SneakyThrows
			void shouldCallRestClient() {
				repository.find();

				verify(restClient).performRequest(any());
			}

			@Test
			@SneakyThrows
			void shouldReturnEmptyOptional() {
				when(httpEntity.getContentLength()).thenReturn(Long.valueOf(EMPTY_CONTENT.length()));
				when(httpEntity.getContent()).thenReturn(new ByteArrayInputStream(EMPTY_CONTENT.getBytes()));
				when(response.getEntity()).thenReturn(httpEntity);
				when(restClient.performRequest(any())).thenReturn(response);

				var result = repository.find();

				assertThat(result).isEmpty();
			}

			@Test
			void shouldReturnMetaDocument() {
				var result = repository.find();

				assertThat(result).isPresent();
			}
		}
	}

	@DisplayName("Test handling exceptions")
	@Nested
	class TestExceptionHandling {
		@BeforeEach
		@SneakyThrows
		void init() {
			when(restClient.performRequest(any())).thenThrow(ResponseException.class);
		}

		@Test
		@SneakyThrows
		void shouldHandleExceptionOnInsert() {
			assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> repository.insert(MetaDocumentTestFactory.create())); // NOSONAR
		}

		@Test
		@SneakyThrows
		void shouldHandleExceptionOnUpdate() {
			assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> repository.update(MetaDocumentTestFactory.create())); // NOSONAR
		}

		@Test
		@SneakyThrows
		void shouldHandleExceptionOnLoad() {
			assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> repository.find());
		}
	}

	@DisplayName("Test creating requests")
	@Nested
	class TestCreatingRequests {

		@BeforeEach
		void init() {
			ReflectionTestUtils.setField(repository, "indexName", ZUFI_MANAGER);
		}

		@DisplayName("verify update request")
		@Nested
		class VerifyUpdateRequest {
			@Test
			void shouldCreateUpdateRequest() {
				var request = repository.createUpdateRequest(MetaDocumentTestFactory.create());

				assertThat(request.getEndpoint())
						.isEqualTo(MetaDocumentRepository.INDEX_UPDATE_TEMPLATE.formatted(ZUFI_MANAGER, MetaDocumentTestFactory.ID));
			}

			@Test
			void shouldSetPath() {
				var request = repository.createUpdateRequest(MetaDocumentTestFactory.create());

				assertThat(request.getEndpoint())
						.isEqualTo(MetaDocumentRepository.INDEX_UPDATE_TEMPLATE.formatted(ZUFI_MANAGER, MetaDocumentTestFactory.ID));
			}

		}

		@DisplayName("verify insert request")
		@Nested
		class VerifyInsertRequest {
			@Test
			void shouldCreateInsertRequest() {
				var request = repository.createInsertRequest(MetaDocumentTestFactory.createBuilder().id(null).build());

				assertThat(request.getMethod()).isEqualTo(HttpMethod.PUT);
			}

			@Test
			void shouldSetPath() {
				var request = repository.createInsertRequest(MetaDocumentTestFactory.create());

				assertThat(request.getEndpoint())
						.isEqualTo(MetaDocumentRepository.INDEX_INSERT_TEMPLATE.formatted(ZUFI_MANAGER, MetaDocumentTestFactory.ID));
			}
		}

		@DisplayName("verify created request body")
		@Nested
		class VerifyRequestBody {
			@Test
			void shouldSetId() {
				var body = createBody();

				assertThat(body.getId()).isEqualTo(MetaDocumentTestFactory.ID);
			}

			@Test
			void shouldSetLastRead() {
				var body = createBody();

				assertThat(body.getLastRead()).isEqualTo(MetaDocumentTestFactory.LAST_READ);
			}

			@Test
			void shouldSetNaechsterIndex() {
				var body = createBody();

				assertThat(body.getNaechsterIndex()).isEqualTo(MetaDocumentTestFactory.NAECHSTER_INDEX);
			}

			@Test
			void shouldSetLastReadComplete() {
				var body = createBody();

				assertThat(body.isLastReadComplete()).isEqualTo(MetaDocumentTestFactory.LAST_READ_COMPLETE);
			}

			@Test
			void shouldSetNaechsteAnfrageUrl() {
				var body = createBody();

				assertThat(body.getNaechsteAnfrageUrl()).isEqualTo(MetaDocumentTestFactory.NAECHSTE_ANFRAGE_URL);
			}

			@Test
			void shouldSetObjectType() {
				var body = createBody();

				assertThat(body.getObjectType()).isEqualTo(MetaDocument.class.getName());
			}

			private MetaDocument createBody() {
				return getBody(repository.createInsertRequest(MetaDocumentTestFactory.create()));
			}

			@SneakyThrows
			private MetaDocument getBody(Request request) {
				var contentStream = new ByteArrayOutputStream();
				IOUtils.copy(request.getEntity().getContent(), contentStream);
				return new ObjectMapper().readValue(contentStream.toByteArray(), MetaDocument.class);
			}
		}
	}
}
