/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.metadocument;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.zufi.pvog.AntwortTransfernachrichtTestFactory;

class MetaDocumentServiceTest {
	@Spy
	@InjectMocks
	MetaDocumentService metaDocumentService;

	@Mock
	MetaDocumentRepository metaDocumentRepository;

	@DisplayName("Test handling of the MetaDocument")
	@Nested
	class TestMetaDocument {

		private final MetaDocument metaDocument = MetaDocumentTestFactory.create();

		@DisplayName("verify insering the document")
		@Nested
		class VerifyInserting {
			@Test
			void shouldInsertNewDocument() {
				metaDocumentService.insert(metaDocument);

				verify(metaDocumentRepository).insert(metaDocument);
			}
		}

		@DisplayName("verify updating the document")
		@Nested
		class VerifyUpdating {
			@Test
			void shouldUpdatedDocument() {
				metaDocumentService.update(metaDocument);

				verify(metaDocumentRepository).update(metaDocument);
			}
		}

		@DisplayName("verify loading the document")
		@Nested
		class VerifyLoading {
			@Test
			void shouldFindTheDocument() {
				metaDocumentService.find();

				verify(metaDocumentRepository).find();
			}

			@Test
			void shouldFindNaechsterIndex() {
				metaDocumentService.findNaechsterIndex();

				verify(metaDocumentRepository).find();
			}
		}
	}

	@DisplayName("verify updating the MetaDocument")
	@Nested
	class VerifyUpdateMetaDocument {
		@Test
		void shouldSetNaechsteAnfrageUrl() {
			var updated = updateMetaDocument();

			assertThat(updated.getNaechsteAnfrageUrl()).isEqualTo(AntwortTransfernachrichtTestFactory.NEXT_URL);
		}

		@Test
		void shouldSetNaechsterIndex() {
			var updated = updateMetaDocument();

			assertThat(updated.getNaechsterIndex()).isEqualTo(AntwortTransfernachrichtTestFactory.NEXT_INDEX);
		}

		@Test
		void shouldSetLastReadComplete() {
			var updated = updateMetaDocument();

			assertThat(updated.isLastReadComplete()).isEqualTo(AntwortTransfernachrichtTestFactory.VOLLSTAENDIG);
		}

		@Test
		void shouldSetLastRead() {
			var updated = updateMetaDocument();

			assertThat(updated.getLastRead()).isNotEqualTo(MetaDocumentTestFactory.LAST_READ);
		}

		@Test
		void shouldSetId() {
			var updated = metaDocumentService.updateMetaDocument(MetaDocumentTestFactory.createBuilder().id(null).build(),
					AntwortTransfernachrichtTestFactory.create());

			assertThat(updated.getId()).isNotNull();
		}

		private MetaDocument updateMetaDocument() {
			return metaDocumentService.updateMetaDocument(MetaDocumentTestFactory.create(), AntwortTransfernachrichtTestFactory.create());
		}
	}

	@DisplayName("verify saving the MetaDocument")
	@Nested
	class VerifySavingMetaDocument {
		@Test
		void shouldUpdate() {
			when(metaDocumentRepository.find()).thenReturn(Optional.of(MetaDocumentTestFactory.create()));

			metaDocumentService.saveMetaDocument(AntwortTransfernachrichtTestFactory.create());

			verify(metaDocumentRepository).update(any());
		}

		@Test
		void shouldInsert() {
			when(metaDocumentRepository.find()).thenReturn(Optional.empty());

			metaDocumentService.saveMetaDocument(AntwortTransfernachrichtTestFactory.create());

			verify(metaDocumentRepository).insert(any());
		}
	}
}
