/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;

import java.io.IOException;
import java.util.Objects;

import jakarta.inject.Inject;

import org.apache.http.util.EntityUtils;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.RestClient;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.zufi.common.HttpMethod;
import io.quarkus.test.junit.QuarkusTest;
import lombok.SneakyThrows;

@QuarkusTest
public class OrganisationsEinheitRepositoryITCase {
	private static final String INDEX_PATH = "/zufi-manager";
	private static final String REFRESH_PATH = INDEX_PATH + "/_refresh";
	private static final String SEARCH_TEMPLATE = INDEX_PATH + "/_search";
	private static final String SEARCH_BODY_TEMPLATE = "{\"query\": {\"match\": { \"id\": \"%s\"}}}";

	@Inject
	OrganisationsEinheitRepository organisationsEinheitRepository;

	@Inject
	RestClient elasticClient;

	@DisplayName("Test inserting OrganisationsEinheit objects")
	@Nested
	class TestInserting {
		@Test
		void shouldInsert() {
			deleteIndex();
			var data = organisationsEinheitRepository.upsert(OrganisationsEinheitTestFactory.create());

			refresh();
			var responseEntity = searchById(data.getId(), true);
			assertThat(responseEntity).isNotNull().contains("\"total\":{\"value\":1");
		}
	}

	@DisplayName("Test deleting OrganisationsEinheit objects")
	@Nested
	class TestDeleting {
		String id;

		@BeforeEach
		void initDatabase() {
			deleteIndex();
			id = organisationsEinheitRepository.upsert(OrganisationsEinheitTestFactory.create()).getId();
			refresh();
		}

		@Test
		void shouldDelete() {
			organisationsEinheitRepository.delete(id);

			refresh();
			var responseEntity = searchById(id, false);
			assertThat(responseEntity).isNotNull().contains("\"total\":{\"value\":0");
		}

	}

	void deleteIndex() {
		try {
			elasticClient.performRequest(new Request("DELETE", INDEX_PATH));
		} catch (IOException e) {
			// ignore
		}
	}

	@SneakyThrows
	String searchById(String id, boolean pollForResult) {
		var request = new Request(HttpMethod.GET, SEARCH_TEMPLATE);
		request.setJsonEntity(SEARCH_BODY_TEMPLATE.formatted(id));

		var response = elasticClient.performRequest(request);

		if (pollForResult) {
			int count = 0;
			while (Objects.nonNull(response.getEntity()) && count < 1000) {
				response = elasticClient.performRequest(request);
				count++;
			}
		}
		return EntityUtils.toString(response.getEntity());
	}

	@SneakyThrows
	void refresh() {
		elasticClient.performRequest(new Request(HttpMethod.POST, REFRESH_PATH));
	}
}
