/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static de.ozgcloud.zufi.organisationseinheit.OrganisationsEinheitRepository.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.apache.commons.io.IOUtils;
import org.apache.http.StatusLine;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.ResponseException;
import org.elasticsearch.client.RestClient;
import org.jboss.resteasy.reactive.common.util.RestMediaType;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.util.ReflectionTestUtils;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.zufi.common.HttpMethod;
import lombok.SneakyThrows;

class OrganisationsEinheitRepositoryTest {
	private static final String ZUFI_MANAGER = "zufi-manager";
	private static final int HTTP_CREATED = 201;
	private static final int HTTP_OK = 200;

	@Spy
	@InjectMocks
	OrganisationsEinheitRepository repository;

	@Mock
	RestClient elasticClient;

	@Mock
	Response response;

	@Mock
	StatusLine statusLine;

	@BeforeEach
	void initIndexName() {
		ReflectionTestUtils.setField(repository, "indexName", ZUFI_MANAGER);
	}

	@DisplayName("Test inserting/updating or deleteing OrganisationsEinheit documents")
	@Nested
	class TestOrganisationsEinheitRepository {

		@DisplayName("when inserting or updating a OrganisationsEinheit document")
		@Nested
		class VerifyUpserting {
			@Test
			@SneakyThrows
			void shouldCallClientOnInsert() {
				mockResponse(HTTP_CREATED);
				var organisationsEinheit = OrganisationsEinheitTestFactory.create();

				var data = repository.upsert(organisationsEinheit);

				verify(elasticClient).performRequest(any(Request.class));
				assertThat(data).isNotNull().usingRecursiveComparison().isEqualTo(organisationsEinheit);
				verify(statusLine, never()).getReasonPhrase();
			}

			@Test
			@SneakyThrows
			void shouldCallClientOnUpdate() {
				mockResponse(HTTP_OK);
				var organisationsEinheit = OrganisationsEinheitTestFactory.create();

				var data = repository.upsert(organisationsEinheit);

				verify(elasticClient).performRequest(any(Request.class));
				assertThat(data).isNotNull().usingRecursiveComparison().isEqualTo(organisationsEinheit);
				verify(statusLine, never()).getReasonPhrase();
			}

			@Test
			@SneakyThrows
			void shouldHandleUnexpectedResponseStatus() {
				mockResponse(300);

				repository.upsert(OrganisationsEinheitTestFactory.create());

				verify(statusLine).getReasonPhrase();
			}

			private void mockResponse(int status) throws IOException {
				when(statusLine.getStatusCode()).thenReturn(status);
				when(response.getStatusLine()).thenReturn(statusLine);
				when(elasticClient.performRequest(any())).thenReturn(response);
			}

			@DisplayName("verify insert request")
			@Nested
			class VerifyInsertRequest {
				@Test
				void shouldHaveRequest() {
					var request = createRequest();

					assertThat(request).isNotNull();
				}

				@Test
				void shouldHaveMethodPost() {
					var request = createRequest();

					assertThat(request.getMethod()).isEqualTo(HttpMethod.POST);
				}

				@Test
				void shouldHaveEndpoint() {
					var request = createRequest();

					assertThat(request.getEndpoint())
							.isEqualTo(INDEX_UPSERT_TEMPLATE.formatted(ZUFI_MANAGER, OrganisationsEinheitTestFactory.ID));
				}

				@Test
				@SneakyThrows
				void shouldHaveBody() {
					var request = createRequest();

					assertThat(request.getEntity()).isNotNull();
				}

				@DisplayName("verfiy request entity")
				@Nested
				class VerifyRequestEntity {
					@Test
					void shouldHaveContentType() {
						var request = createRequest();

						assertThat(request.getEntity().getContentType().getValue()).isNotNull().isEqualTo(
								RestMediaType.APPLICATION_JSON + "; charset=UTF-8");
						assertThat(request.getEntity().getContentType().getName()).isNotNull().isEqualTo(
								"Content-Type");
					}

					@Test
					@SneakyThrows
					void shouldHaveBody() {
						var data = OrganisationsEinheitTestFactory.create();
						var request = repository.createInsertRequest(data);
						var expectedDataString = new ObjectMapper().writeValueAsString(data);
						var baos = new ByteArrayOutputStream();
						IOUtils.copy(request.getEntity().getContent(), baos);
						var dataString = new String(baos.toByteArray());

						assertThat(dataString).isEqualTo(expectedDataString);
					}
				}

				private Request createRequest() {
					return repository.createInsertRequest(OrganisationsEinheitTestFactory.create());
				}
			}
		}

		@DisplayName("when deleting a OrganisationsEinheit document")
		@Nested
		class VerifyDeleting {
			@Test
			@SneakyThrows
			void shouldCallClient() {
				when(statusLine.getStatusCode()).thenReturn(HTTP_OK);
				when(response.getStatusLine()).thenReturn(statusLine);
				when(elasticClient.performRequest(any())).thenReturn(response);

				repository.delete(OrganisationsEinheitTestFactory.ID);

				verify(elasticClient).performRequest(any(Request.class));
			}

			@DisplayName("verify delete request")
			@Nested
			class VerifyRequest {
				@Test
				void shouldCreateRequest() {
					var request = repository.createDeleteRequest(OrganisationsEinheitTestFactory.ID);

					assertThat(request).isNotNull();
				}

				@Test
				void shouldHavePostMethod() {
					var request = repository.createDeleteRequest(OrganisationsEinheitTestFactory.ID);

					assertThat(request.getMethod()).isEqualTo(HttpMethod.POST);
				}

				@Test
				void shouldHaveEndpoint() {
					var request = repository.createDeleteRequest(OrganisationsEinheitTestFactory.ID);

					assertThat(request.getEndpoint()).isEqualTo(DELETE_ENTRY_TEMPLATE.formatted(ZUFI_MANAGER));
				}

				@DisplayName("verfiy request entity")
				@Nested
				class VerifyRequestEntity {
					@Test
					void shouldHaveContentType() {
						var request = repository.createDeleteRequest(OrganisationsEinheitTestFactory.ID);

						assertThat(request.getEntity().getContentType().getValue()).isNotNull().isEqualTo(
								RestMediaType.APPLICATION_JSON + "; charset=UTF-8");
						assertThat(request.getEntity().getContentType().getName()).isNotNull().isEqualTo(
								"Content-Type");
					}

					@Test
					@SneakyThrows
					void shouldHaveBody() {
						var data = OrganisationsEinheitTestFactory.create();
						var request = repository.createDeleteRequest(data.getId());
						var baos = new ByteArrayOutputStream();
						IOUtils.copy(request.getEntity().getContent(), baos);
						var dataString = new String(baos.toByteArray());

						assertThat(dataString).isEqualTo(DELETE_BODY_TEMPLATE.formatted(data.getId()));
					}
				}
			}
		}

		@DisplayName("verify exception handling")
		@Nested
		class OnException {
			@Test
			@SneakyThrows
			void shouldHandleException() {
				when(elasticClient.performRequest(any())).thenThrow(ResponseException.class);

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> repository.upsert(OrganisationsEinheitTestFactory.create())); // NOSONAR
			}
		}
	}
}
