/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.zufi.pvog.TransferOperationEntryTestFactory;

class OrganisationsEinheitServiceTest {
	@Spy
	@InjectMocks
	OrganisationsEinheitService service;

	@Mock
	OrganisationsEinheitMapper mapper;

	@Mock
	OrganisationsEinheitRepository repository;

	@DisplayName("Test processing transfer objects")
	@Nested
	class TestProcessTransferOperationEntry {
		@Test
		void shouldCallRespoitoryWithSchreibeObjekt() {
			when(mapper.mapPayload(any())).thenReturn(OrganisationsEinheitTestFactory.create());

			service.process(TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry());

			verify(repository).upsert(any(OrganisationsEinheit.class));
		}

		@Test
		void shouldCallRepositoryWithLoescheObjekt() {
			service.process(TransferOperationEntryTestFactory.createDeleteEntry());

			verify(repository).delete(anyString());
		}

		@Test
		void shouldNotCallRepository() {
			var entry = TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry();

			service.process(entry.toBuilder().operationType(null).build());

			verify(repository, never()).upsert(any(OrganisationsEinheit.class));
			verify(repository, never()).delete(anyString());
		}
	}

	@DisplayName("Test inserting a SchreibeObjekt")
	@Nested
	class TestHandleSchreibeObjekt {
		@BeforeEach
		void init() {
			when(mapper.mapPayload(any())).thenReturn(OrganisationsEinheitTestFactory.create());
		}

		@Test
		void shouldCallService() {
			service.insert(TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry());

			verify(repository).upsert(any(OrganisationsEinheit.class));
		}
	}

	@DisplayName("Test handling a LoescheObjekt")
	@Nested
	class TestHandlingLoescheObjekt {
		@Test
		void shouldCallService() {
			service.delete(TransferOperationEntryTestFactory.createDeleteEntry());

			verify(repository).delete(any(String.class));
		}
	}

	@DisplayName("Test validation of SchreibeObjekt")
	@Nested
	class TestValidation {
		@Test
		void shouldBeValid() {
			assertThat(service.isValid(TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry())).isTrue();
		}

		@Test
		void shouldBeInvalidBecauseOfInvalidOperationType() {
			assertThat(service.isValid(TransferOperationEntryTestFactory.createDeleteEntry())).isFalse();
		}

		@Test
		void shouldBeInvalidBecauseOfMissingPayload() {
			assertThat(service.isValid(TransferOperationEntryTestFactory.createBuilder().payload(Map.of()).build())).isFalse();
		}
	}
}
