/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.io.StringReader;

import jakarta.inject.Inject;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import org.apache.http.util.EntityUtils;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.zufi.schema.TransferOperation040502;
import de.ozgcloud.zufi.common.HttpMethod;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.mockito.InjectMock;
import lombok.SneakyThrows;

@QuarkusTest
class PvogProcessorITCase {
	private static final String INDEX_PATH = "/zufi-manager";
	private static final String REFRESH_PATH = INDEX_PATH + "/_refresh";
	private static final String SEARCH_TEMPLATE = INDEX_PATH + "/_search";
	private static final String SEARCH_BODY = "{\"query\": {\"match\": { \"objectType\": \"%s\"}}}";

	@Inject
	PvogProcessor processor;

	@InjectMock
	PvogService pvogService;

	@Inject
	RestClient elasticClient;
	
	@Inject
	TransferOperationMapper mapper;

	@DisplayName("Test processing PVOG data")
	@Nested
	class TestPvogProcessor {
		@BeforeEach
		void prepareIndex() {
			deleteIndex();
			createIndex();
		}

		@DisplayName("Test inserting PVOG data")
		@Nested
		class TestPvogProcessorSchreibeObjekte {
			@BeforeEach
			void initRemoteService() {
				when(pvogService.getAllVerwaltungsObjekte()).thenReturn(AntwortTransfernachrichtTestFactory.create());
				when(pvogService.getIncrementVerwaltungsObjekte(anyInt())).thenReturn(AntwortTransfernachrichtTestFactory.createBuilder()
						.xzufiObjekte(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE_INCREMENT).build());

			}

			@BeforeEach
			@SneakyThrows
			void init() {
				when(pvogService.dataAvailable(any())).thenReturn(true);
				when(pvogService.parseXml(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE))
						.thenReturn(mapper.mapTo(parseXml(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE)));
			}

			@Test
			@SneakyThrows
			void shouldProcessFull() {
				processor.process();

				refresh();
				assertThat(getMetaDocumentReponse()).contains("\"total\":{\"value\":1");
				assertThat(getOrganisationsEinheitResponse())
						.contains("\"total\":{\"value\":" + AntwortTransfernachrichtTestFactory.ANZAHL_ORGANISATIONSEINHEITEN);
			}

			@Test
			@SneakyThrows
			void shouldProcessIncremental() {
				when(pvogService.parseXml(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE_INCREMENT))
						.thenReturn(mapper.mapTo(parseXml(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE_INCREMENT)));

				shouldProcessFull();

				processor.process();

				refresh();
				assertThat(getMetaDocumentReponse()).contains("\"total\":{\"value\":1");
				assertThat(getOrganisationsEinheitResponse())
						.contains("\"total\":{\"value\":" + (AntwortTransfernachrichtTestFactory.ANZAHL_ORGANISATIONSEINHEITEN + 1));
			}
		}

		@DisplayName("Test deleting PVOG data")
		@Nested
		class TestPvogProcessorLoescheObjekte {
			private static final String KIEL_ID = "10324180";

			@BeforeEach
			@SneakyThrows
			void initData() {
				when(pvogService.getAllVerwaltungsObjekte()).thenReturn(AntwortTransfernachrichtTestFactory.create());
				when(pvogService.getIncrementVerwaltungsObjekte(anyInt()))
						.thenReturn(AntwortTransfernachrichtTestFactory.createBuilder().xzufiObjekte(createLoescheTransferObject()).build());
				when(pvogService.parseXml(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE))
						.thenReturn(mapper.mapTo(parseXml(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE)));
				when(pvogService.parseXml(createLoescheTransferObject()))
						.thenReturn(mapper.mapTo(parseXml(createLoescheTransferObject())));
				when(pvogService.dataAvailable(any())).thenReturn(true);

				processor.process();

				refresh();
			}

			private String createLoescheTransferObject() throws JAXBException {
				var transferOperation = new TransferOperation040502();
				transferOperation.getTransferOperation().add(
						LoescheObjektTestFactory.createWithId(KIEL_ID));
				JAXBContext jc = JAXBContext.newInstance("de.ozgcloud.zufi.schema");
				Marshaller m = jc.createMarshaller();
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				m.marshal(transferOperation, baos);

				return new String(baos.toByteArray());
			}

			@Test
			@SneakyThrows
			void shouldProcess() {
				processor.process();

				refresh();

				assertThat(getMetaDocumentReponse()).contains("\"total\":{\"value\":1");
				assertThat(getOrganisationsEinheitResponse()).doesNotContain(KIEL_ID);
			}
		}

		@SneakyThrows
		TransferOperation040502 parseXml(String xzufiObjekte) {
			var jaxbUnmarshaller = JAXBContext.newInstance("de.ozgcloud.zufi.schema").createUnmarshaller();

			return (TransferOperation040502) jaxbUnmarshaller.unmarshal(new StringReader(xzufiObjekte));
		}

		@SneakyThrows
		String getMetaDocumentReponse() {
			return EntityUtils.toString(getSearchResponse("de.ozgcloud.zufi.metadocument.MetaDocument").getEntity());
		}

		@SneakyThrows
		String getOrganisationsEinheitResponse() {
			return EntityUtils.toString(getSearchResponse("de.ozgcloud.zufi.organisationseinheit.OrganisationsEinheit").getEntity());
		}

		@SneakyThrows
		Response getSearchResponse(String className) {
			var request = new Request(HttpMethod.GET, SEARCH_TEMPLATE.formatted(INDEX_PATH));
			request.setJsonEntity(SEARCH_BODY.formatted(className));
			return elasticClient.performRequest(request);
		}

		void deleteIndex() {
			try {
				elasticClient.performRequest(new Request(HttpMethod.DELETE, INDEX_PATH));
			} catch (Exception e) {
				// ignore exceptions
			}
		}

		@SneakyThrows
		void createIndex() {
			elasticClient.performRequest(new Request(HttpMethod.PUT, INDEX_PATH));
		}

		@SneakyThrows
		void refresh() {
			elasticClient.performRequest(new Request(HttpMethod.POST, REFRESH_PATH));
		}
	}
}
