/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Optional;

import javax.xml.bind.JAXBException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.zufi.schema.TransferOperation040502;
import de.ozgcloud.zufi.metadocument.MetaDocumentService;
import de.ozgcloud.zufi.metadocument.MetaDocumentTestFactory;
import de.ozgcloud.zufi.organisationseinheit.OrganisationsEinheitService;
import lombok.SneakyThrows;

class PvogProcessorTest {
	@Spy
	@InjectMocks
	PvogProcessor processor;
	@Mock
	PvogService service;
	@Mock
	MetaDocumentService metaDocumentService;
	@Mock
	OrganisationsEinheitService organisationsEinheitService;

	@DisplayName("Test processing the PVOG data")
	@Nested
	class TestProcessor {
		@DisplayName("when it is a full sync")
		@Nested
		class OnFullSync {
			@BeforeEach
			@SneakyThrows
			void init() {
				when(service.dataAvailable(any())).thenReturn(true);
				when(service.getAllVerwaltungsObjekte()).thenReturn(AntwortTransfernachrichtTestFactory.create());
				when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.empty());
				when(service.parseXml(anyString())).thenReturn(TransportOperationTestFactory.create());
			}

			@Test
			void shouldCallPvogServiceGetAllVerwaltungsObjekte() {
				processor.process();

				verify(service).getAllVerwaltungsObjekte();
			}

			@Test
			void shouldCallMetaDocumentServiceSaveForInsert() {
				processor.process();

				verify(metaDocumentService).saveMetaDocument(any());
			}

		}

		@DisplayName("when it is a incremental sync")
		@Nested
		class OnIncrementalSync {
			@BeforeEach
			@SneakyThrows
			void init() {
				when(service.dataAvailable(any())).thenReturn(true);
				when(service.getIncrementVerwaltungsObjekte(anyInt())).thenReturn(AntwortTransfernachrichtTestFactory.create());
				when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.of(MetaDocumentTestFactory.NAECHSTER_INDEX));
				when(service.parseXml(anyString())).thenReturn(TransportOperationTestFactory.create());
			}

			@Test
			void shouldCallPvogServiceGetVerwaltungsObjekteIncrement() {
				processor.process();

				verify(service).getIncrementVerwaltungsObjekte(anyInt());
			}

			@Test
			void shouldCallMetaDocumentServiceSaveForUpdate() {
				processor.process();

				verify(metaDocumentService).saveMetaDocument(any());
			}
		}

		@Test
		void shouldCallParseXml() throws Exception {
			service.parseXml(AntwortTransfernachrichtTestFactory.create().xzufiObjekte());

			verify(service).parseXml(any());
		}

		@DisplayName("Processing incomming data")
		@Nested
		class WhenProcessingData {
			private TransferOperationEntry schreibeObject1 = TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry();
			private TransferOperationEntry schreibeObject2 = TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry();
			private TransferOperationEntry schreibeObject3 = TransferOperationEntryTestFactory.createWriteOrganisationsEinheitEntry();
			private TransferOperationEntry loescheObject1 = TransferOperationEntryTestFactory.createDeleteEntry();

			@Mock
			TransferOperation040502 transferOperation;

			@Test
			void shouldProcessInOrder() {
				var op = new TransferOperation.TransferOperationBuilder()
						.entries(List.of(schreibeObject1, schreibeObject2, loescheObject1, schreibeObject3)).build();

				processor.processTransferOperations(op);

				verify(organisationsEinheitService).process(schreibeObject1);
				verify(organisationsEinheitService).process(schreibeObject2);
				verify(organisationsEinheitService).process(loescheObject1);
				verify(organisationsEinheitService).process(schreibeObject3);
			}

			@Test
			void shouldHandleLoescheObjekt() {
				var op = new TransferOperation.TransferOperationBuilder().entries(List.of(loescheObject1)).build();

				processor.processTransferOperations(op);

				verify(organisationsEinheitService).process(any(TransferOperationEntry.class));
			}

			@Test
			void shouldHandleSchreibeObjekt() {
				var op = new TransferOperation.TransferOperationBuilder().entries(List.of(schreibeObject1)).build();

				processor.processTransferOperations(op);

				verify(organisationsEinheitService).process(any(TransferOperationEntry.class));
			}

			@DisplayName("verify that received data is valid")
			@Nested
			class VerifyProcessingParsedData {
				@BeforeEach
				@SneakyThrows
				void init() {
					when(service.dataAvailable(any())).thenReturn(true);
					when(service.parseXml(anyString())).thenReturn(TransportOperationTestFactory.create());
				}

				@Test
				void shouldCallProcessOperationsOnceOnFullSync() {
					when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.empty());
					when(service.getAllVerwaltungsObjekte()).thenReturn(AntwortTransfernachrichtTestFactory.create());

					processor.process();

					verify(processor).processTransferOperations(any());
				}

				@Test
				void shouldCallProcessOperationsOnceOnIncrementalSync() {
					when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.of(MetaDocumentTestFactory.NAECHSTER_INDEX));
					when(service.getIncrementVerwaltungsObjekte(anyInt())).thenReturn(AntwortTransfernachrichtTestFactory.create());

					processor.process();

					verify(processor).processTransferOperations(any());
				}

				@Test
				void shouldCallProcessOperationsTwice() {
					when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.empty());
					var first = AntwortTransfernachrichtTestFactory.createBuilder().vollstaendig(false).build();
					when(service.getAllVerwaltungsObjekte()).thenReturn(first);
					var second = AntwortTransfernachrichtTestFactory.create();
					when(service.getIncrementVerwaltungsObjekte(anyInt())).thenReturn(second);
					when(service.hasMoreData(first)).thenReturn(true);
					when(service.hasMoreData(second)).thenReturn(false);

					processor.process();

					verify(processor, times(2)).processTransferOperations(any());
				}

			}

			@DisplayName("when no updates are available")
			@Nested
			class OnEmptyData {
				@Test
				@SneakyThrows
				void shouldNotParseDataOnEmptyData() {
					when(metaDocumentService.findNaechsterIndex()).thenReturn(Optional.of(MetaDocumentTestFactory.NAECHSTER_INDEX));
					when(service.getIncrementVerwaltungsObjekte(anyInt())).thenReturn(AntwortTransfernachrichtTestFactory.create());
					when(service.dataAvailable(any())).thenReturn(false);

					processor.process();

					verify(service).getIncrementVerwaltungsObjekte(anyInt());
					verify(service, never()).parseXml(anyString());
				}
			}

			@DisplayName("check error handling")
			@Nested
			class OnException {
				@BeforeEach
				void init() {
					when(service.dataAvailable(any())).thenReturn(true);
				}

				@Test
				@SneakyThrows
				void shouldThrowTechnicalExceptionOnParseError() {
					doThrow(new JAXBException("Error")).when(service).parseXml(anyString());

					assertThatExceptionOfType(TechnicalException.class).isThrownBy( // NOSONAR
							() -> processor.handleAntwortTransfernachricht(
									AntwortTransfernachrichtTestFactory.create()))
							.withMessageContaining(PvogProcessor.INVALID_DATA_RECEICED_FROM_PVOG_SERVICE);

					verify(metaDocumentService, never()).saveMetaDocument(any());
				}

				@Test
				@SneakyThrows
				void shouldThrowTechnicalExceptionOnInvalidXml() {
					doThrow(new JAXBException("Error")).when(service).parseXml(anyString());

					assertThatExceptionOfType(TechnicalException.class).isThrownBy( // NOSONAR
							() -> processor.handleAntwortTransfernachricht(
									AntwortTransfernachrichtTestFactory.createBuilder()
											.xzufiObjekte(AntwortTransfernachrichtTestFactory.BROKEN_XZUFI_OBJEKTE).build()))
							.withMessageContaining(PvogProcessor.INVALID_DATA_RECEICED_FROM_PVOG_SERVICE);

					verify(metaDocumentService, never()).saveMetaDocument(any());
				}

				@Test
				@SneakyThrows
				void shouldThrowTechnicalExceptionOnError() {
					doThrow(new RuntimeException()).when(processor).processTransferOperations(any());

					assertThatExceptionOfType(TechnicalException.class).isThrownBy( // NOSONAR
							() -> processor.handleAntwortTransfernachricht(
									AntwortTransfernachrichtTestFactory.create()))
							.withMessageContaining(PvogProcessor.GENERAL_ERROR_PROCESSING_PVOG_DATA);

					verify(metaDocumentService, never()).saveMetaDocument(any());
				}
			}
		}
	}
}
