/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import static de.ozgcloud.zufi.pvog.AntwortTransfernachrichtTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import org.eclipse.microprofile.rest.client.inject.RestClient;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.common.errorhandling.TechnicalException;
import io.quarkus.test.common.QuarkusTestResource;
import io.quarkus.test.junit.QuarkusTest;

@QuarkusTest
@QuarkusTestResource(WireMockExtension.class)
class PvogRemoteServiceITCase {
	private static final int START_INDEX = 0;

	@RestClient
	PvogRemoteService remoteService;

	@DisplayName("Get AntwortTransfernachricht from PVOG service")
	@Nested
	class TestLoadingData {
		@Test
		void shouldLoad() {
			AntwortTransfernachricht reply = remoteService.getVerwaltungsObjekte(START_INDEX, ARS);

			assertThat(reply).isNotNull();
		}

		@Test
		void shouldHandleServerError() {
			assertThatExceptionOfType(TechnicalException.class)
					.isThrownBy(() -> remoteService.getVerwaltungsObjekte(START_INDEX, ERROR_ARS))
					.withMessageContaining("500");
		}

		@Test
		void shouldHandleNotFoundError() {
			assertThatExceptionOfType(TechnicalException.class)
					.isThrownBy(() -> remoteService.getVerwaltungsObjekte(START_INDEX, "xxx"))
					.withMessageContaining("404");
		}

		@DisplayName("verify received data")
		@Nested
		class CheckResponse {

			@Test
			void shouldHaveAnzahlObjekte() {
				var response = remoteService.getVerwaltungsObjekte(START_INDEX, ARS);

				assertThat(response).isNotNull()
						.hasFieldOrPropertyWithValue(ANZAHL_OBJEKTE_FIELD, ANZAHL_OBJEKTE);
			}

			@Test
			void shouldHaveNaechsterIndex() {
				var response = remoteService.getVerwaltungsObjekte(START_INDEX, ARS);

				assertThat(response).isNotNull()
						.hasFieldOrPropertyWithValue(NEXT_INDEX_FIELD, NEXT_INDEX);
			}

			@Test
			void shouldHaveNaechsteAnfrageUrl() {
				var response = remoteService.getVerwaltungsObjekte(START_INDEX, ARS);

				assertThat(response).isNotNull()
						.hasFieldOrPropertyWithValue(NEXT_URL_FIELD, NEXT_URL);
			}

			@Test
			void shouldHaveVollstaendig() {
				var response = remoteService.getVerwaltungsObjekte(START_INDEX, ARS);

				assertThat(response).isNotNull()
						.hasFieldOrPropertyWithValue(VOLLSTAENDIG_FIELD, VOLLSTAENDIG);
			}

			@Test
			void shouldHaveXzufiObjekte() {
				var response = remoteService.getVerwaltungsObjekte(START_INDEX, ARS);

				assertThat(response.xzufiObjekte()).isNotNull();
			}

			@Test
			void shouldHaveDeserialzedXml() {
				var response = remoteService.getVerwaltungsObjekte(START_INDEX, ARS);

				assertThat(response.xzufiObjekte()).doesNotContain("\\\"");
			}
		}
	}
}
