/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import static de.ozgcloud.zufi.pvog.AntwortTransfernachrichtTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import javax.xml.bind.JAXBException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.util.ReflectionTestUtils;

import lombok.SneakyThrows;

class PvogServiceTest {

	@Spy
	@InjectMocks
	PvogService service;
	@Mock
	PvogRemoteService remoteService;

	@DisplayName("Get all verwaltungsobjekte of")
	@Nested
	class TestGetAllVerwaltungsObjekteOf {

		@BeforeEach
		void init() {
			ReflectionTestUtils.setField(service, "remoteService", remoteService);

			when(remoteService.getVerwaltungsObjekte(anyInt(), any())).thenReturn(AntwortTransfernachrichtTestFactory.create());
		}

		@Test
		void shouldCallClient() {
			service.getAllVerwaltungsObjekte();

			verify(remoteService).getVerwaltungsObjekte(anyInt(), any());
		}

		@Test
		void shouldHaveAnzahlObjekte() {
			var response = getVerwaltungsObjekte();

			assertThat(response).isNotNull()
					.hasFieldOrPropertyWithValue(ANZAHL_OBJEKTE_FIELD, ANZAHL_OBJEKTE);
		}

		@Test
		void shouldHaveNaechsterIndex() {
			var response = getVerwaltungsObjekte();

			assertThat(response).isNotNull()
					.hasFieldOrPropertyWithValue(NEXT_INDEX_FIELD, NEXT_INDEX);
		}

		@Test
		void shouldHaveNaechsteAnfrageUrl() {
			var response = getVerwaltungsObjekte();

			assertThat(response).isNotNull()
					.hasFieldOrPropertyWithValue(NEXT_URL_FIELD, NEXT_URL);
		}

		@Test
		void shouldHaveVollstaendig() {
			var response = getVerwaltungsObjekte();

			assertThat(response).isNotNull()
					.hasFieldOrPropertyWithValue(VOLLSTAENDIG_FIELD, VOLLSTAENDIG);
		}

		@Test
		void shouldHaveXzufiObjekte() {
			var response = getVerwaltungsObjekte();

			assertThat(response).isNotNull()
					.hasFieldOrPropertyWithValue(XZUFI_OBJEKTE_FIELD, XZUFI_OBJEKTE);
		}

		private AntwortTransfernachricht getVerwaltungsObjekte() {
			return service.getAllVerwaltungsObjekte();
		}
	}

	@DisplayName("Get verwaltungsobjekte inkrement of")
	@Nested
	class TestGetVerwaltungsObjekteInkrementOf {

		@Test
		void shouldCallClientForIncrement() {
			service.getIncrementVerwaltungsObjekte(anyInt());

			verify(remoteService).getVerwaltungsObjekte(anyInt(), any());
		}
	}

	@DisplayName("Test if there is more dat ato fetch from the remote service")
	@Nested
	class TestHasMoreData {

		@Test
		void shouldReturnTrue() {
			var res = service.hasMoreData(AntwortTransfernachrichtTestFactory.createBuilder().vollstaendig(false).build());

			assertThat(res).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var res = service.hasMoreData(AntwortTransfernachrichtTestFactory.create());

			assertThat(res).isFalse();
		}
	}

	@DisplayName("Throw JAXBException when parsing invalid xml")
	@Nested
	class TestHandlingInvalidXmlData {
		@Test
		@SneakyThrows
		void shouldThrowExceptionOnInvalidXml() {
			assertThatExceptionOfType(JAXBException.class).isThrownBy( // NOSONAR
					() -> service.parseXml(AntwortTransfernachrichtTestFactory.BROKEN_XZUFI_OBJEKTE));
		}
	}

	@DisplayName("Verify the xzufi data of the incomming AntwortTransfernachricht")
	@Nested
	class TestVerifyingAntwortTransfernachricht {
		@Test
		void shouldHandleEmptyBody() {
			assertThat(
					service.dataAvailable(AntwortTransfernachrichtTestFactory.createBuilder().xzufiObjekte("").build()))
							.isFalse();
		}

		@Test
		void shouldHandleNullBody() {
			assertThat(
					service.dataAvailable(AntwortTransfernachrichtTestFactory.createBuilder().xzufiObjekte(null).build()))
							.isFalse();
		}

		@Test
		@SneakyThrows
		void shouldReturnTrue() {
			assertThat(service.dataAvailable(AntwortTransfernachrichtTestFactory.create())).isTrue();
		}
	}
}