/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.zufi.common.OperationType;
import de.ozgcloud.zufi.common.PayloadType;

class TransferOperationEntryMapperTest {
	TransferOperationEntryMapper mapper = Mappers.getMapper(TransferOperationEntryMapper.class);

	@DisplayName("Test mapping of SchreibeObjekte of type OrganisationsEinheit")
	@Nested
	class TestMappingOrganisationsEinheitSchreibeObjekte {
		@Test
		void shouldMapOperationType() {
			var op = map();

			assertThat(op.getOperationType()).isEqualTo(OperationType.WRITE);
		}

		@Test
		void shouldMapPayloadType() {
			var op = map();

			assertThat(op.getPayloadType()).isEqualTo(PayloadType.ORGANISATIONSEINHEIT);
		}

		@Test
		void shouldMapEntryWithNameOnly() {
			var op = mapper.mapTo(SchreibeObjektTestFactory.createWithoutAnschrift());

			assertThat(op.getPayloadType()).isEqualTo(PayloadType.ORGANISATIONSEINHEIT);
			assertThat(op.getPayload()).doesNotContainKey(TransferOperationEntry.ORGANISATIONSEINHEIT_BUNDESLAND_FIELD);
		}

		@DisplayName("verify mapped payload")
		@Nested
		class VerifyPayload {
			@Test
			void shouldHaveId() {
				var op = map();

				assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ID_FIELD, IdentifikatorTestFactory.VALUE);
			}

			@Test
			void shouldHaveBezirk() {
				var op = map();

				assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ORGANISATIONSEINHEIT_BEZIRK_FIELD,
						CodeBezirkDestatisTestFactory.NAME);
			}

			@Test
			void shouldHaveBundesland() {
				var op = map();

				assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ORGANISATIONSEINHEIT_BUNDESLAND_FIELD,
						CodeBundeslandDestatisTestFactory.NAME);
			}

			@Test
			void shouldHaveGemeindeschluessel() {
				var op = map();

				assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ORGANISATIONSEINHEIT_GEMEINDESCHLUESSEL_FIELD,
						CodeGemeindeschluesselDestatisTestFactory.NAME);
			}

			@Test
			void shouldHaveGemeindeTeilschluessel() {
				var op = map();

				assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ORGANISATIONSEINHEIT_GEMEINDETEILSCHLUESSEL_FIELD,
						CodeGemeindeteilschluesselTestFactory.NAME);
			}

			@Test
			void shouldHaveKreis() {
				var op = map();

				assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ORGANISATIONSEINHEIT_KREIS_FIELD, CodeKreisDestatisTestFactory.NAME);
			}

			@Test
			void shouldHaveRegionalschluessel() {
				var op = map();

				assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ORGANISATIONSEINHEIT_REGIONALSCHLUESSEL_FIELD,
						CodeRegionalschluesselDestatisTestFactory.NAME);
			}
		}

		private TransferOperationEntry map() {
			return mapper.mapTo(SchreibeObjektTestFactory.create());
		}
	}

	@DisplayName("Test mapping of SchreibeObjekte of without OrganisationsEinheit data")
	@Nested
	class TestMappingSchreibeObjekte {
		@Test
		void shouldMapEmptyOrgansationEinheit() {
			var op = map();

			assertThat(op.getPayload()).isNull();
		}

		@Test
		void shouldMapPayloadType() {
			var op = map();

			assertThat(op.getPayloadType()).isEqualTo(PayloadType.UNKNOWN);
		}

		@Test
		void shouldMapOperationType() {
			var op = map();

			assertThat(op.getOperationType()).isEqualTo(OperationType.WRITE);
		}

		private TransferOperationEntry map() {
			return mapper.mapTo(SchreibeObjektTestFactory.createLeistungObjekteOnly());
		}
	}

	@DisplayName("Test mapping of LoescheObjekte")
	@Nested
	class TestMappingLoescheObjekt {
		@Test
		void shouldMapOperationType() {
			var op = map();

			assertThat(op.getOperationType()).isEqualTo(OperationType.DELETE);
		}

		@Test
		void shouldMapPayloadType() {
			var op = map();

			assertThat(op.getPayloadType()).isNull();
		}

		@Test
		void shouldMapId() {
			var op = map();

			assertThat(op.getPayload()).containsEntry(TransferOperationEntry.ID_FIELD, IdentifikatorTestFactory.VALUE);
		}

		private TransferOperationEntry map() {
			return mapper.mapTo(LoescheObjektTestFactory.create());
		}
	}
}
