/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.pvog;

import static org.assertj.core.api.Assertions.*;

import java.io.ByteArrayOutputStream;
import java.io.StringReader;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.zufi.schema.TransferOperation040502;
import de.ozgcloud.zufi.common.OperationType;
import de.ozgcloud.zufi.common.PayloadType;
import lombok.SneakyThrows;

class TransferOperationMapperTest {
	@Spy
	@InjectMocks
	TransferOperationMapper mapper = Mappers.getMapper(TransferOperationMapper.class);

	@Spy
	TransferOperationEntryMapper entryMapper = Mappers.getMapper(TransferOperationEntryMapper.class);

	@DisplayName("Verify the received transfer object")
	@Nested
	class TestMappedData {
		@Test
		void shouldHaveEntries() {
			var data = map();

			assertThat(data.getEntries()).hasSize(AntwortTransfernachrichtTestFactory.ANZAHL_OBJEKTE);
		}

		@DisplayName("Verify the mapped schreibe data object")
		@Nested
		class VerifySchreibeObjekt {
			@Test
			void shouldHaveOperationWrite() {
				var data = map();

				assertThat(data.getEntries().get(0).getOperationType()).isEqualTo(OperationType.WRITE);
			}

			@Test
			void shouldHavePayloadType() {
				var data = map();

				assertThat(data.getEntries().get(0).getPayloadType()).isEqualTo(PayloadType.ORGANISATIONSEINHEIT);
			}

			@Test
			void shouldPayloadHaveFields() {
				var data = map();

				var payload = data.getEntries().get(0).getPayload();

				assertThat(payload).containsKeys(
						TransferOperationEntry.ID_FIELD,
						TransferOperationEntry.ORGANISATIONSEINHEIT_BUNDESLAND_FIELD,
						TransferOperationEntry.ORGANISATIONSEINHEIT_NAME_FIELD,
						TransferOperationEntry.ORGANISATIONSEINHEIT_REGIONALSCHLUESSEL_FIELD);
			}
		}

		@DisplayName("Verify the mapped loesche data object")
		@Nested
		class VerifyLoescheObjekt {
			private static final String KIEL_ID = "10324180";

			@Test
			void shouldHaveOperationDelete() {
				var data = map();

				assertThat(data.getEntries().get(0).getOperationType()).isEqualTo(OperationType.DELETE);
			}

			@Test
			void shouldNotHavePayloadType() {
				var data = map();

				assertThat(data.getEntries().get(0).getPayloadType()).isNull();
			}

			@Test
			void shouldPayloadHaveFields() {
				var data = map();

				var payload = data.getEntries().get(0).getPayload();
				assertThat(payload).containsKeys(TransferOperationEntry.ID_FIELD);
			}

			private TransferOperation map() {
				return mapper.mapTo(parseXml(createLoescheTransferObject()));
			}

			@SneakyThrows
			private String createLoescheTransferObject() {
				var transferOperation = new TransferOperation040502();
				transferOperation.getTransferOperation().add(
						LoescheObjektTestFactory.createWithId(KIEL_ID));
				JAXBContext jc = JAXBContext.newInstance("de.ozgcloud.zufi.schema");
				Marshaller m = jc.createMarshaller();
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				m.marshal(transferOperation, baos);

				return new String(baos.toByteArray());
			}
		}
	}

	private TransferOperation map() {
		return mapper.mapTo(parseXml(AntwortTransfernachrichtTestFactory.XZUFI_OBJEKTE));
	}

	@SneakyThrows
	TransferOperation040502 parseXml(String xzufiObjekte) {
		var jaxbUnmarshaller = JAXBContext.newInstance("de.ozgcloud.zufi.schema").createUnmarshaller();

		return (TransferOperation040502) jaxbUnmarshaller.unmarshal(new StringReader(xzufiObjekte));
	}
}
