/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.search;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.security.KeyStore;
import java.security.cert.Certificate;

import javax.net.ssl.SSLContext;

import org.apache.http.impl.nio.client.HttpAsyncClientBuilder;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.util.ReflectionUtils;

import de.ozgcloud.common.errorhandling.TechnicalException;

class SSLContextConfiguratorTest {

	@Spy
	private SSLContextConfigurator configurator = new SSLContextConfigurator();

	@DisplayName("Customize http client")
	@Nested
	class TestCustomizeHttpClient {

		@DisplayName("on https")
		@Nested
		class TestOnHttps {

			@Mock
			private HttpAsyncClientBuilder builder;
			@Mock
			private SSLContext sslContext;

			@BeforeEach
			void init() {
				doReturn(sslContext).when(configurator).buildSslContext();
				mockElasticSearchProtocol("https");
			}

			@Test
			void shouldSetSSLContextOnHttps() {
				configurator.customizeHttpClient(builder);

				verify(builder).setSSLContext(any());
			}

			@Test
			void shouldCallBuildSslContext() {
				configurator.customizeHttpClient(builder);

				verify(configurator).buildSslContext();
			}

			private void mockElasticSearchProtocol(String value) {
				var field = ReflectionUtils.findField(SSLContextConfigurator.class, "elasticsearchProtocol");
				field.setAccessible(true);
				ReflectionUtils.setField(field, configurator, value);
			}

		}

		@DisplayName("build ssl context")
		@Nested
		class TestBuildSslContext {

			@Test
			void shouldThrowTechnicalExceptionOnException() {
				doThrow(new RuntimeException()).when(configurator).buildKeyStore();

				assertThatThrownBy(() -> configurator.buildSslContext())
						.isInstanceOf(TechnicalException.class);
			}
		}

		@DisplayName("build key store")
		@Nested
		class TestBuildKeyStore {

			@Mock
			private Certificate certificate;
			@Mock
			private KeyStore keyStore;

			@DisplayName("should call")
			@Nested
			class TestCall {

				@BeforeEach
				void init() {
					doReturn(certificate).when(configurator).generateCertificate();
					doReturn(keyStore).when(configurator).initKeyStore();
				}

				@Test
				void initKeyStore() {
					configurator.buildKeyStore();

					verify(configurator).initKeyStore();
				}

				@Test
				void generateCertificate() {
					configurator.buildKeyStore();

					verify(configurator).generateCertificate();
				}
			}

			@Test
			void shouldThrowTechnicalExceptionOnException() {
				doThrow(new RuntimeException()).when(configurator).initKeyStore();

				assertThatThrownBy(() -> configurator.buildKeyStore())
						.isInstanceOf(TechnicalException.class);
			}
		}
	}

	@DisplayName("generate certificate")
	@Nested
	class TestGenerateCertificate {

		@Test
		void shouldThrowTechnicalExceptionOnException() {
			doThrow(new RuntimeException()).when(configurator).loadCaCrt();

			assertThatThrownBy(() -> configurator.generateCertificate())
					.isInstanceOf(TechnicalException.class);
		}
	}
}