/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.common.callcontext;

import java.nio.charset.StandardCharsets;
import java.util.Optional;

import org.apache.logging.log4j.CloseableThreadContext;

import io.grpc.ForwardingServerCallListener;
import io.grpc.Metadata;
import io.grpc.Metadata.Key;
import io.grpc.ServerCall;
import io.grpc.ServerCall.Listener;
import io.grpc.ServerCallHandler;
import io.grpc.ServerInterceptor;
import net.devh.boot.grpc.server.interceptor.GrpcGlobalServerInterceptor;

@GrpcGlobalServerInterceptor
class EnhanceGrpcLoggingInterceptor implements ServerInterceptor {

	private static final String REQUEST_ID_KEY = "requestId";

	static final String KEY_REQUEST_ID = "REQUEST_ID-bin";
	static final String REQUEST_ID_PLACEHOLDER = "no-requestId-given";

	@Override
	public <A, B> Listener<A> interceptCall(ServerCall<A, B> call, Metadata headers, ServerCallHandler<A, B> next) {
		return new LogContextSettingListener<>(next.startCall(call, headers), headers);
	}

	class LogContextSettingListener<A> extends ForwardingServerCallListener.SimpleForwardingServerCallListener<A> {
		private final String requestId;
		private final Metadata headers;

		public LogContextSettingListener(ServerCall.Listener<A> delegate, Metadata headers) {
			super(delegate);
			this.headers = headers;
			this.requestId = getRequestId();
		}

		@Override
		public void onMessage(A message) {
			doSurroundOn(() -> super.onMessage(message));
		}

		@Override
		public void onHalfClose() {
			doSurroundOn(super::onHalfClose);
		}

		@Override
		public void onCancel() {
			doSurroundOn(super::onCancel);
		}

		@Override
		public void onComplete() {
			doSurroundOn(super::onComplete);
		}

		@Override
		public void onReady() {
			doSurroundOn(super::onReady);
		}

		void doSurroundOn(Runnable runnable) {
			try (CloseableThreadContext.Instance ignored = CloseableThreadContext.put(REQUEST_ID_KEY, requestId)) {
				runnable.run();
			}
		}

		String getRequestId() {
			return getFromHeaders(KEY_REQUEST_ID, headers).orElseGet(() -> REQUEST_ID_PLACEHOLDER);
		}
	}

	public static Optional<String> getFromHeaders(String key, Metadata headers) {
		return Optional.ofNullable(headers.get(createKeyOf(key))).map(val -> new String(val, StandardCharsets.UTF_8));
	}

	public static Key<byte[]> createKeyOf(String key) {
		return Key.of(key, Metadata.BINARY_BYTE_MARSHALLER);
	}
}
