/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.common.errorhandling;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.common.errorhandling.TechnicalException;
import io.grpc.Metadata;
import io.grpc.Metadata.Key;
import io.grpc.Status.Code;
import io.grpc.StatusException;

class GrpcExceptionHandlersTest {

	private static final String LOG_MESSAGE = "A Test Exception";
	@Spy
	@InjectMocks
	private GrpcExceptionHandlers handler;

	@Nested
	class TestHandleTechnicalException {

		@Test
		void shouldHaveStatusCode() {
			var status = handleException(createTechnicalException()).getStatus();

			assertThat(status.getCode()).isEqualTo(Code.INTERNAL);
		}

		@Test
		void shouldHaveDescription() {
			var status = handleException(createTechnicalException()).getStatus();

			assertThat(status.getDescription()).contains(LOG_MESSAGE);
		}

		@Test
		void shouldHaveExceptionId() {
			var exception = createTechnicalException();

			var metaData = handleException(exception).getTrailers();

			assertThat(metaData.get(createExceptionIdKey())).isEqualTo(exception.getExceptionId());
		}

		private TechnicalException createTechnicalException() {
			return new TechnicalException(LOG_MESSAGE, new RuntimeException());
		}

		private StatusException handleException(TechnicalException exception) {
			return handler.handleTechnicalException(exception);
		}
	}

	@Nested
	class TestHandleRuntimeException {

		final String exceptionId = "42";
		final RuntimeException exception = new RuntimeException(LOG_MESSAGE);

		@BeforeEach
		void mockExceptionId() {
			doReturn(exceptionId).when(handler).createExceptionId();
		}

		@Test
		void shouldHaveStatusCode() {
			var status = handleException(exception).getStatus();

			assertThat(status.getCode()).isEqualTo(Code.INTERNAL);
		}

		@Test
		void shouldHaveMessage() {
			var statusException = handleException(exception);

			assertThat(statusException.getMessage()).contains(LOG_MESSAGE);
			assertThat(statusException.getMessage()).contains(exceptionId);
		}

		@Test
		void shouldHaveStatusDescription() {
			var status = handleException(exception).getStatus();

			assertThat(status.getDescription()).contains(LOG_MESSAGE);
			assertThat(status.getDescription()).contains(exceptionId);
		}

		@Test
		void shouldHaveExceptionId() {
			var metaData = handleException(exception).getTrailers();

			assertThat(metaData.get(createExceptionIdKey())).isEqualTo(exceptionId);
		}

		private StatusException handleException(RuntimeException exception) {
			return handler.handleRuntimeException(exception);
		}
	}

	private Key<String> createExceptionIdKey() {
		return createKeyOf(GrpcExceptionHandlers.KEY_EXCEPTION_ID);
	}

	private Key<String> createKeyOf(String key) {
		return Key.of(key, Metadata.ASCII_STRING_MARSHALLER);
	}
}