/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static de.ozgcloud.zufi.organisationseinheit.OrganisationsEinheitTestDocument.*;
import static org.assertj.core.api.Assertions.*;

import java.util.UUID;

import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.core.Map;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.elasticsearch.core.ElasticsearchOperations;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.test.context.ContextConfiguration;

import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.zufi.SearchTestInitializer;
import lombok.SneakyThrows;

@ITCase
@ContextConfiguration(initializers = SearchTestInitializer.class)
class OrganisationsEinheitCustomRepositoryITCase {
	@Autowired
	private OrganisationsEinheitCustomRepository repository;

	@Autowired
	private RestHighLevelClient elasticsearchClient;

	@Autowired
	private ElasticsearchOperations elasticsearchOperations;

	@BeforeEach
	void initDatabase() throws Exception {
		elasticsearchOperations.indexOps(IndexCoordinates.of("zufi-manager")).delete();

		var insert = new IndexRequest().index("zufi-manager").source(Map.of(ID_KEY, ID, VALUE_KEY, VALUE)).id(ID);
		elasticsearchClient.index(insert, RequestOptions.DEFAULT);
	}

	@DisplayName("Test updating an Organisationseinheit document in elasticsearch")
	@Nested
	class TestUpdate {

		@Test
		void shouldReturnUpdateSuccess() {
			var updateSuccess = repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS);

			assertThat(updateSuccess).isTrue();
		}

		@Test
		void shouldDoUpdate() {
			var updateSuccess = repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS);

			assertThat(updateSuccess).isTrue();
			var doc = elasticsearchOperations.get(ID, OrganisationsEinheitTestDocument.class);
			assertThat(doc)
					.hasFieldOrPropertyWithValue(VORGANG_MANAGER_ADDRESS_KEY, VORGANG_MANAGER_ADDRESS);
		}

		@Test
		@SneakyThrows
		void shouldDoUpdateSameExistingAddress() {
			var insert = new IndexRequest().index("zufi-manager")
					.source(Map.of(ID_KEY, ID, VALUE_KEY, VALUE, VORGANG_MANAGER_ADDRESS_KEY, VORGANG_MANAGER_ADDRESS)).id(ID);
			elasticsearchClient.index(insert, RequestOptions.DEFAULT);

			var updateSuccess = repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS);

			assertThat(updateSuccess).isTrue();
			var doc = elasticsearchOperations.get(ID, OrganisationsEinheitTestDocument.class);
			assertThat(doc)
					.hasFieldOrPropertyWithValue(VORGANG_MANAGER_ADDRESS_KEY, VORGANG_MANAGER_ADDRESS);
		}

		@Test
		void shouldOnlyAddServiceAddress() {
			repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS);

			var doc = elasticsearchOperations.get(ID, OrganisationsEinheitTestDocument.class);
			assertThat(doc)
					.hasFieldOrPropertyWithValue(ID_KEY, ID)
					.hasFieldOrPropertyWithValue(VALUE_KEY, VALUE)
					.hasFieldOrPropertyWithValue(VORGANG_MANAGER_ADDRESS_KEY, VORGANG_MANAGER_ADDRESS);
		}

		@Test
		void shouldNotUpdate() {
			var updateSuccess = repository.updateServiceAddress(UUID.randomUUID().toString(), VORGANG_MANAGER_ADDRESS);

			assertThat(updateSuccess).isFalse();
		}
	}
}
