/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.zufi.organisationseinheit;

import static de.ozgcloud.zufi.organisationseinheit.OrganisationsEinheitTestDocument.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;

import org.elasticsearch.ElasticsearchStatusException;
import org.elasticsearch.action.DocWriteResponse.Result;
import org.elasticsearch.action.update.UpdateRequest;
import org.elasticsearch.action.update.UpdateResponse;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.SneakyThrows;

class OrganisationsEinheitCustomRepositoyTest {
	@Spy
	@InjectMocks
	private OrganisationsEinheitCustomRepositoryImpl repository;

	@Mock
	private ElasticsearchClientFacade client;

	@BeforeEach
	void init() {
		ReflectionTestUtils.setField(repository, "indexName", "test-index");
	}

	@DisplayName("Test updating elasticsearch entry")
	@Nested
	class TestUpdate {

		@Mock
		private UpdateResponse response;

		@Test
		@SneakyThrows
		void shouldUpdate() {
			doReturn(response).when(client).update(any(UpdateRequest.class));

			repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS);

			verify(client).update(any(UpdateRequest.class));
		}

		@DisplayName("Test successfull results")
		@Nested
		class TestSuccess {
			@ParameterizedTest
			@SneakyThrows
			@EnumSource(value = Result.class, names = { "UPDATED", "NOOP" })
			void shouldHandleSuccessResult(Result result) {
				when(response.getResult()).thenReturn(result);
				doReturn(response).when(client).update(any(UpdateRequest.class));

				var res = repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS);

				assertThat(res).isTrue();
			}
		}

		@DisplayName("Test unsuccessfull results")
		@Nested
		class TestFailures {
			@ParameterizedTest
			@SneakyThrows
			@EnumSource(value = Result.class, names = { "NOT_FOUND", "DELETED" })
			void shouldHandleNotFoundResult(Result result) {
				when(response.getResult()).thenReturn(result);
				doReturn(response).when(client).update(any(UpdateRequest.class));

				var res = repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS);

				assertThat(res).isFalse();
			}
		}

		@DisplayName("Test handling exception while updating elasticsearch entry")
		@Nested
		class TestExceptions {
			@Test
			@SneakyThrows
			void shouldHandleIOException() {
				when(client.update(any(UpdateRequest.class))).thenThrow(IOException.class);

				assertThatExceptionOfType(TechnicalException.class).isThrownBy(() -> repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS));
			}

			@Test
			@SneakyThrows
			void shouldNotThrowExceptionOnElasticsearchStatusException() {
				when(client.update(any(UpdateRequest.class))).thenThrow(ElasticsearchStatusException.class);

				assertThatNoException().isThrownBy(() -> repository.updateServiceAddress(ID, VORGANG_MANAGER_ADDRESS));
			}
		}
	}
}
